// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <utility>

#include "Common.hpp"
#include "Context.hpp"
#include "ContextManager.hpp"
#include "Debug.hpp"
#include "Runtime.hpp"

template <LogSubsystem logSubsystem>
class Blockable : public Logger<logSubsystem> {
 protected:
	Runtime& runtime;

	ContextManager& contextManager;

	Blockable(Runtime& runtime) : runtime(runtime), contextManager(runtime.getContextManager()) {}

	// Older clang-tidy versions show a
	// performance-unnecessary-value-param, newer versions do not show
	// this error if std::move() is used (as we do).
	// NOLINTNEXTLINE(performance-unnecessary-value-param)
	void block(func_t freshContextHook) {
		LOGD("block() blockedContext is " << Context::getCurrentContext());
		contextManager.saveAndStartNew(std::move(freshContextHook));
	}

	void unblock(Context* context) {
		assert(context != nullptr);
		// cppcheck-suppress unsafeClassCanLeak
		Fiber* unblockFiber =
				Fiber::from([this, context]() { contextManager.discardAndResume(context); });
		runtime.schedule(*unblockFiber);
	}
};
