[Section .text]

[GLOBAL switch_and_load_context]
[GLOBAL save_and_switch_context]
[GLOBAL switch_context]

; Stack layout after the state was saved on the stack.
;
;		| ... |
;	 40 | r15 |
;	 32 | r14 |
;	 24 | r13 |
;	 16 | r12 |
;	  8 | rbp |
;	  0 | rbx | <- ToS after switching stack to this context
%macro STATE_SAVE 0
	sub rsp, 48
	mov [rsp+40], r15
	mov [rsp+32], r14
	mov [rsp+24], r13
	mov [rsp+16], r12
	mov [rsp+8],  rbp
	mov [rsp],    rbx
%endmacro

%macro STATE_LOAD 0
	mov rbx, [rsp]
	mov rbp, [rsp+8]
	mov r12, [rsp+16]
	mov r13, [rsp+24]
	mov r14, [rsp+32]
	mov r15, [rsp+40]
	add rsp, 48
%endmacro

; switch_and_load_context(void *to)
;                         rdi
align 16
switch_and_load_context:
	; Load new context.
	mov rsp, [rdi] ; saved rsp
	STATE_LOAD
	ret

; save_and_switch_context(void** savedSpNew, void** savedSpOld)
;                         rdi                rsi
align 16
save_and_switch_context:
	; Store current context.
	STATE_SAVE
	mov [rsi], rsp ; save rsp
	; Fall through

; switch_context(void *contextTos)
;                rdi
switch_context:
	; Switch to new context: Activate the new stack by setting rsp,
	; which must point to a word which represent the memory address
	; where we want to continue.
	mov rsp, [rdi]
	; Pop the memory address where to continue from the
	; stack. Which means: Continue at [rsp].
	ret
