// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#include "Debug.hpp"

#include <iostream>	 // for operator<<, cerr, ostream, basic_ostream
#include <mutex>		 // for mutex, unique_lock

#include "Runtime.hpp"		 // for Runtime
#include "emper-common.h"	 // for workerid_t

static std::mutex worker_log_mutex;

void worker_log(const std::string& prefix, const std::string& message) {
	const workerid_t workerId = Runtime::getWorkerId();

	std::unique_lock<std::mutex> lock(worker_log_mutex);
	std::cerr << (unsigned int)workerId;
	if (!prefix.empty()) {
		std::cerr << " " << prefix << " ";
	} else {
		std::cerr << "       ";
	}
	std::cerr << message << std::endl;
}
