// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <functional>	 // for function

#include "Debug.hpp"			 // for LogSubsystem, LogSubsystem::SCHED, Logger
#include "Fiber.hpp"			 // for Fiber
#include "emper-common.h"	 // for workeraffinity_t

class Runtime;

class Scheduler : public Logger<LogSubsystem::SCHED> {
 protected:
	Runtime& runtime;
	Scheduler(Runtime& runtime);

	void addNewWorkerHook(const std::function<void(void)>& hook);

	static inline auto getAffinityBuffer(Fiber& fiber) -> workeraffinity_t* {
		return fiber.getAffinityBuffer();
	}

	static inline void increaseRefCount(Fiber& fiber) { fiber.doAtomicIncrRefCount(); }

	void notifyRuntimeAboutNewWork();

 public:
	virtual void schedule(Fiber& fiber) = 0;

	virtual auto nextFiber() -> Fiber* = 0;
};
