// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#include <new>	// for operator new

#include "BinaryPrivateSemaphore.hpp"		 // for BinaryPrivateSemaphore, BPS
#include "CountingPrivateSemaphore.hpp"	 // for CountingPrivateSemaphore, CPS
#include "Fiber.hpp"										 // for Fiber, Fiber::NOT_AFFINE
#include "Runtime.hpp"									 // for Runtime
#include "emper-common.h"								 // for workeraffinity_t
#include "emper.h"											 // for fiber, cps, bps, runtime

auto init_runtime(void) -> runtime* {
	auto* r = new Runtime();
	return reinterpret_cast<runtime*>(r);
}

void wait_until_runtime_finished() {
	Runtime* r = Runtime::getRuntime();
	r->waitUntilFinished();
}

auto fiber_from(void (*function)(void*), void* arg) -> fiber* {
	Fiber* f = Fiber::from(function, arg);
	return reinterpret_cast<fiber*>(f);
}

auto fiber_from0(void (*function)()) -> fiber* {
	Fiber* f = Fiber::from(function);
	return reinterpret_cast<fiber*>(f);
}

auto aff_fiber_from(void (*function)(void*), void* arg, workeraffinity_t* affinity) -> fiber* {
	Fiber* f = Fiber::from(function, arg, affinity);
	return reinterpret_cast<fiber*>(f);
}

auto aff_fiber_from0(void (*function)(), workeraffinity_t* affinity) -> fiber* {
	Fiber* f = Fiber::from(function, affinity);
	return reinterpret_cast<fiber*>(f);
}

void init_affinity(workeraffinity_t affinity[], unsigned int n) {
	for (unsigned int i = 0; i < n; ++i) {
		affinity[i] = Fiber::NOT_AFFINE;
	}
}

void schedule(fiber* fiber) {
	Runtime* runtime = Runtime::getRuntime();
	auto* f = reinterpret_cast<Fiber*>(fiber);
	runtime->schedule(*f);
}

auto new_binary_sem() -> bps* {
	BPS* sem = new BPS();
	return reinterpret_cast<bps*>(sem);
}

void signal_bps(bps* sem) {
	auto* binaryPrivateSemaphore = reinterpret_cast<BinaryPrivateSemaphore*>(sem);
	binaryPrivateSemaphore->signal();
}

void signal_and_exit_bps(bps* sem) {
	auto* binaryPrivateSemaphore = reinterpret_cast<BinaryPrivateSemaphore*>(sem);
	binaryPrivateSemaphore->signalAndExit();
}

void wait_bps(bps* sem) {
	auto* binaryPrivateSemaphore = reinterpret_cast<BinaryPrivateSemaphore*>(sem);
	binaryPrivateSemaphore->wait();
}

auto new_counting_sem() -> cps* {
	CPS* sem = new CPS();
	return reinterpret_cast<cps*>(sem);
}

auto new_counting_sem_with(unsigned int i) -> cps* {
	CPS* sem = new CPS(i);
	return reinterpret_cast<cps*>(sem);
}

void signal_cps(cps* sem) {
	auto* countingPrivateSemaphore = reinterpret_cast<CountingPrivateSemaphore*>(sem);
	countingPrivateSemaphore->signal();
}

void signal_and_exit_cps(cps* sem) {
	auto* countingPrivateSemaphore = reinterpret_cast<CountingPrivateSemaphore*>(sem);
	countingPrivateSemaphore->signalAndExit();
}

void wait_cps(cps* sem) {
	auto* countingPrivateSemaphore = reinterpret_cast<CountingPrivateSemaphore*>(sem);
	countingPrivateSemaphore->wait();
}
