// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <deque>
#include <mutex>

namespace adt {

template <typename I>
class LockedUnboundedQueue {
 private:
	std::mutex queue_mutex;

	std::deque<I*> deque;

 public:
	void enqueue(I* item) {
		std::lock_guard<std::mutex> lock(queue_mutex);
		deque.push_front(item);
	}

	auto dequeue() -> I* {
		std::lock_guard<std::mutex> lock(queue_mutex);
		if (deque.empty()) {
			return nullptr;
		}
		I* res = deque.back();
		deque.pop_back();
		return res;
	}
};
}	 // namespace adt
