// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <atomic>
#include <cstdint>
#include <memory>

#include "RuntimeStrategy.hpp"

class Dispatcher;
class LawsDispatcher;
class LawsScheduler;
class LawsStrategyStats;
class Runtime;
class RuntimeStrategyStats;
class Scheduler;

class LawsStrategy : public RuntimeStrategy {
 private:
	enum struct FiberSource : unsigned int {
		fromPriority,
		fromLocal,
		stolen,
		mainThread,
	};

	std::atomic<std::uint64_t> scheduledFibersToRemotePriority;
	std::atomic<std::uint64_t> scheduledFibersToLocal;
	std::atomic<std::uint64_t> dispatchedFiberFromPriority;
	std::atomic<std::uint64_t> dispatchedFiberFromLocal;
	std::atomic<std::uint64_t> dispatchedFiberStolen;
	std::atomic<std::uint64_t> dispatchedFiberFromMainThread;

	LawsStrategy()
			: scheduledFibersToRemotePriority(0),
				scheduledFibersToLocal(0),
				dispatchedFiberFromPriority(0),
				dispatchedFiberFromLocal(0),
				dispatchedFiberStolen(0),
				dispatchedFiberFromMainThread(0) {}

	auto getScheduler(Runtime& runtime) -> Scheduler& override;

	auto getDispatcher(Runtime& runtime) -> Dispatcher& override;

 public:
	auto getStats() -> std::shared_ptr<RuntimeStrategyStats> override;

	static LawsStrategy INSTANCE;

	friend LawsScheduler;
	friend LawsDispatcher;
	friend LawsStrategyStats;
};
