// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <atomic>
#include <cstdint>
#include <memory>

#include "RuntimeStrategy.hpp"

class Scheduler;
class Dispatcher;
class Runtime;
class RuntimeStrategyStats;
class WsScheduler;
class WsDispatcher;
class WsStrategyStats;

class WsStrategy : public RuntimeStrategy {
 private:
	std::atomic<std::uint64_t> scheduledFibers;
	std::atomic<std::uint64_t> nextFiberFromLocal;
	std::atomic<std::uint64_t> nextFiberStolen;

	WsStrategy() : scheduledFibers(0), nextFiberFromLocal(0), nextFiberStolen(0) {}

	auto getScheduler(Runtime& runtime) -> Scheduler& override;

	auto getDispatcher(Runtime& runtime) -> Dispatcher& override;

 public:
	auto getStats() -> std::shared_ptr<RuntimeStrategyStats> override;

	static WsStrategy INSTANCE;

	friend WsScheduler;
	friend WsDispatcher;
	friend WsStrategyStats;
};
